package webapp

import (
	"crypto/rsa"
	"fmt"
	"io/ioutil"

	"github.com/satori/go.uuid"

	"git.loafle.net/commons_go/logging"
	cwf "git.loafle.net/commons_go/websocket_fasthttp"
	oogw "git.loafle.net/overflow/overflow_gateway_websocket"
	"git.loafle.net/overflow/overflow_gateway_websocket/rpc"
	jwt "github.com/dgrijalva/jwt-go"
	"github.com/valyala/fasthttp"
)

var (
	verifyKey *rsa.PublicKey
	signKey   *rsa.PrivateKey
)

func init() {
	signBytes, err := ioutil.ReadFile("overFlow-private.key")
	if nil != err {
		panic(err)
	}

	signKey, err = jwt.ParseRSAPrivateKeyFromPEM(signBytes)
	if nil != err {
		panic(err)
	}

	verifyBytes, err := ioutil.ReadFile("overFlow-public.pem")
	if nil != err {
		panic(err)
	}

	verifyKey, err = jwt.ParseRSAPublicKeyFromPEM(verifyBytes)
	if nil != err {
		panic(err)
	}
}

func newGatewayRPCHandler() rpc.GatewayRPCHandler {
	gh := &GatewayRPCHandlers{}

	return gh
}

type GatewayRPCHandlers struct {
	rpc.GatewayRPCHandlers
}

func (sh *GatewayRPCHandlers) Init(serverCTX cwf.ServerContext) error {
	return nil
}

func (sh *GatewayRPCHandlers) Handshake(socketCTX cwf.SocketContext, ctx *fasthttp.RequestCtx) (id string, extensionsHeader *fasthttp.ResponseHeader) {
	var ok bool
	// tokenString := string(ctx.Request.Header.Cookie("authToken"))
	// tokenString := "eyJhbGciOiJSUzUxMiIsInR5cCI6IkpXVCJ9.eyJhdWQiOiJ3d3cub3ZlcmZsb3cuY2xvdWQiLCJleHAiOjE1MjA5MzQzMjAsImlhdCI6MTUyMDg0NzkyMCwiaXNzIjoib3ZlckZsb3ciLCJzdWIiOiJvdmVyZmxvd0Bsb2FmbGUuY29tIn0.Umjy3_qvFe4A2wv62r7s2XbqnX_P22HIvAJfx8azM-lejRzYu0wOx9T1UFjRPFzEJO8fEztG1sq3FN5g2097tS6-kbrV5TvB5HclCsK7H384k9CrZEFSgImT-_7yotcPvKKydzvSzLznlbgglr1QwHKH-5JmqyMlxaaYmARmjPHY_tt6NByfeXHEwIL0rh-4TMbx4QOrsBlOHbaRKQqxy8CLwDN2c8-eoEvu1TEIaCxR9rqPrHgsy__1WOY_QY1MdyPzmToXQiwhPyy0hU3jlpf7G9GQOgIEoBD-yrk_WbsFihB2S4uS0TsyopOnEtfSYT5sItnfzCikjNFAaKac39mhW9rPZ-XbxEHK_UYpiCIBtdmiP2ObBCS0FUT2won0V5l4pEw7MTkDehZJptGl45Xrb-ELn-T1lOE6LWuQnZz2sPu2_UHJBcxhPvX6F7cUA3Jm0UGKytq-QOyrUCoiF9YiruDrCXiUkwTQocFwhAGYYeUqQx0KybqLkmn9mkB6zfdyIbY3rk0ep0Kwgp3HTcQR2_6SVkLNAihBpPvZsE5HyVB3hWrOjq-6p0KLj4jfykykj0pA9hNs9qDp5Xb7MR0vRJo__qgvCEe1CiKWwTJY0dXiDDwcIZtqL1u_UdiwZg5Q_O-OCX90IgIETHZNiqAOCKp1IuO9BdplxyCEOBw"
	tokenString := string(ctx.QueryArgs().Peek("authToken"))
	token, err := jwt.Parse(tokenString, func(token *jwt.Token) (interface{}, error) {
		// Don't forget to validate the alg is what you expect:
		if _, ok := token.Method.(*jwt.SigningMethodRSA); !ok {
			return nil, fmt.Errorf("Webapp: Unexpected signing method: %v", token.Header["alg"])
		}

		// hmacSampleSecret is a []byte containing your secret, e.g. []byte("my_secret_key")
		return verifyKey, nil
	})

	if nil != err {
		return "", nil
	}

	var claims jwt.MapClaims
	if claims, ok = token.Claims.(jwt.MapClaims); !ok || !token.Valid {
		logging.Logger().Warnf("Webapp: Token is not valid %v", token)
		return "", nil
	}

	userEmail := claims["sub"].(string)
	sessionID := uuid.NewV4().String()

	socketCTX.SetAttribute(oogw.ClientTypeKey, oogw.MEMBER)
	socketCTX.SetAttribute(oogw.TargetIDKey, userEmail)

	logging.Logger().Debugf("Webapp: Client sessionID %s", sessionID)

	return sessionID, nil
}

func (sh *GatewayRPCHandlers) Validate() {
	sh.GatewayRPCHandlers.Validate()
}
